#ifndef LIBKYNETWORKSEC_H
#define LIBKYNETWORKSEC_H

#ifdef __cplusplus
extern "C" {
#endif

typedef struct kdk_firewall_ctx_t kdk_firewall_ctx;

typedef enum _kdk_firewall_table {
    FW_TABLE_RAW = 0,
    FW_TABLE_MANGLE = 1,
    FW_TABLE_NAT = 2,
    FW_TABLE_FILTER = 3
} kdk_firewall_table;

typedef enum _kdk_firewall_command {
    FW_COMMAND_ADD = 0,
    FW_COMMAND_DEL = 1,
    FW_COMMAND_INSERT = 2,
    FW_COMMAND_NEWCHAIN = 3,
    FW_COMMAND_DELETECHAIN = 4,
    FW_COMMAND_RENAMECHAIN = 5,
    FW_COMMAND_FLUSH = 6
} kdk_firewall_command;

typedef enum _kdk_firewall_chain {
    FW_CHAIN_INPUT = 0,
    FW_CHAIN_FORWARD = 1,
    FW_CHAIN_OUTPUT = 2,
    FW_CHAIN_PREROUTING = 3,
    FW_CHAIN_POSTROUTING = 4
} kdk_firewall_chain;

typedef enum _kdk_firewall_protocol {
    FW_PROTOCOL_TCP = 0,
    FW_PROTOCOL_UDP = 1,
    FW_PROTOCOL_ICMP = 2,
} kdk_firewall_protocol;

typedef enum _kdk_firewall_target {
    FW_TARGET_ACCEPT = 0,
    FW_TARGET_DROP = 1,
    FW_TARGET_REJECT = 2,
} kdk_firewall_target;

/**
 * @brief 申请一个防火墙配置上下文
 * @return kdk_firewall_ctx 上下文指针，需要自行调用kdk_firewall_ctx_free释放此指针内存
 */
extern kdk_firewall_ctx *kdk_firewall_ctx_new();

/**
 * @brief 释放防火墙操作上下文内存
 * @param ctx [in] 使用kdk_firewall_ctx_new申请的
 */
extern void kdk_firewall_ctx_free(kdk_firewall_ctx *ctx);

/**
 * @brief 设置防火墙设置表，包括：raw、mangle、nat、filter
 * @param ctx [in] 防火墙操作上下文
 * @param table [in] 操作表
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_table(kdk_firewall_ctx *ctx, kdk_firewall_table table);

/**
 * @brief 设置防火墙命令项，例如：新增规则、删除规则、清空规则等
 * @param ctx [in] 防火墙操作上下文
 * @param command [in] 命令
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_command(kdk_firewall_ctx *ctx, kdk_firewall_command command);

/**
 * @brief 设置防火墙链，例如：PREROUTING/INPUT/NAT/OUTPUT/POSTROUTING
 * @param ctx [in] 防火墙操作上下文
 * @param chain [in] 链，参见kdk_firewall_chain
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_chain(kdk_firewall_ctx *ctx, kdk_firewall_chain chain);

/**
 * @brief 设置防火墙自定义链
 * @param ctx [in] 防火墙操作上下文
 * @param chain [in] 链，可以是自定义链也可以是原有链
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_chain_ex(kdk_firewall_ctx *ctx, const char *chain);

/**
 * @brief 设置防火墙新旧链名称，此函数主要使用方是更新链名称使用
 * @param ctx [in] 防火墙操作上下文
 * @param oldchain [in] 旧链名称
 * @param newchain [in] 新链名称
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_old_and_new_chain(kdk_firewall_ctx *ctx, const char *oldchain, const char *newchain);

/**
 * @brief 设置防火墙协议
 * @param ctx [in] 防火墙操作上下文
 * @param protocol [in] 协议，参见kdk_firewall_protocol
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_protocol(kdk_firewall_ctx *ctx, kdk_firewall_protocol protocol);

/**
 * @brief 设置防火墙协议，字符串格式的，参见/etc/protocol内容，可以使用非“!”
 * @param ctx [in] 防火墙操作上下文
 * @param protocol [in] 协议，参见/etc/protocol
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_protocol_ex(kdk_firewall_ctx *ctx, const char *protocol);

/**
 * @brief 设置防火墙源地址
 * @param ctx [in] 防火墙操作上下文
 * @param saddr [in] 源地址
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_source_address(kdk_firewall_ctx *ctx, const char *saddr);

/**
 * @brief 设置防火墙规则的目的地址
 * @param ctx [in] 防火墙操作上下文
 * @param daddr [in] 目的地址
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_destnation_address(kdk_firewall_ctx *ctx, const char *daddr);

/**
 * @brief 设置防火墙规则的输入端口
 * @param ctx [in] 防火墙操作上下文
 * @param iface [in] 网络接口，例如：eth0、eth+
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_input_interface(kdk_firewall_ctx *ctx, const char *iface);

/**
 * @brief 设置防火墙规则的输出端口
 * @param ctx [in] 防火墙操作上下文
 * @param iface [in] 网络接口，例如：eth0、eth+
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_output_interface(kdk_firewall_ctx *ctx, const char *iface);

/**
 * @brief 设置防火墙规则的源端口
 * @param ctx [in] 防火墙操作上下文
 * @param port [in] 端口号
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_source_port(kdk_firewall_ctx *ctx, unsigned short port);

/**
 * @brief 设置防火墙规则的源端口范围
 * @param ctx [in] 防火墙操作上下文
 * @param from [in] 端口最小值
 * @param to [in] 端口最大值
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_source_port_range(kdk_firewall_ctx *ctx, unsigned short from, unsigned short to);

/**
 * @brief 使用服务名称设置防火墙规则的源端口
 * @param ctx [in] 防火墙操作上下文
 * @param servicename [in] 服务名称，例如：ssh、ftp等
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_source_port_with_servicename(kdk_firewall_ctx *ctx, const char *servicename);

/**
 * @brief 设置防火墙规则的目的端口
 * @param ctx [in] 防火墙操作上下文
 * @param port [in] 端口号
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_destnation_port(kdk_firewall_ctx *ctx, unsigned short port);

/**
 * @brief 设置防火墙规则的目的端口范围
 * @param ctx [in] 防火墙操作上下文
 * @param from [in] 端口最小值
 * @param to [in] 端口最大值
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_destnation_port_range(kdk_firewall_ctx *ctx, unsigned short from, unsigned short to);

/**
 * @brief 使用服务名称设置防火墙规则的目的端口
 * @param ctx [in] 防火墙操作上下文
 * @param servicename [in] 服务名称，例如：ssh、ftp等
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_destnation_port_with_servicename(kdk_firewall_ctx *ctx, const char *servicename);

/**
 * @brief 设置防火墙规则的目的地
 * @param ctx [in] 防火墙操作上下文
 * @param target [in] 目的地，参见kdk_firewall_target
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_target(kdk_firewall_ctx *ctx, kdk_firewall_target target);

/**
 * @brief 设置防火墙的目的地
 * @param ctx [in] 防火墙操作上下文
 * @param target [in] 目的地，可以是原有目的地REJECT/ACCEPT/DROP，也可以是自定义链
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_set_target_ex(kdk_firewall_ctx *ctx, const char *target);

/**
 * @brief 下发防护墙策略
 * @param ctx [in] 防火墙操作上下文
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_issue_policy(kdk_firewall_ctx *ctx);

/**
 * custom chain operation
 * 自定义链操作
 */

/**
 * @brief 创建一个自定义链
 * @param chain [in] 链
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_create_custom_chain(const char *chain);

/**
 * @brief 删除一个自定义链
 * @param chain [in] 链
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_delete_custom_chain(const char *chain);

/**
 * @brief 重命名一个自定义链
 * @param oldchain [in] 原有链
 * @param newchain [in] 新链
 * @return int 返回0表示成功，返回非零表示失败
 */
extern int kdk_firewall_rename_custom_chain(const char *oldchain, const char *newchain);

#ifdef __cplusplus
}
#endif

#endif//LIBKYNETWORKSEC_H
